@@ -1,4 +1,6 @@
-{-# OPTIONS_GHC -fno-warn-unused-imports #-} -- Monoid required < 7.9
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE CPP              #-}
+{-# LANGUAGE ConstraintKinds  #-}
 {- |
 'Foldable' functions, with wrappers like the "Safe" module.
 -}
@@ -19,18 +21,24 @@ module Safe.Foldable(
 
 import Safe.Util
 import Data.Foldable as F
-import Data.Monoid
 import Data.Maybe
+import Data.Monoid
+import Prelude
+import Safe.Partial
 
 
 ---------------------------------------------------------------------
 -- UTILITIES
 
+fromNote :: Partial => String -> String -> Maybe a -> a
 fromNote = fromNoteModule "Safe.Foldable"
 
 isNull :: Foldable t => t a -> Bool
+#if __GLASGOW_HASKELL__ < 710
 isNull = null . toList
-
+#else
+isNull = F.null
+#endif
 
 ---------------------------------------------------------------------
 -- WRAPPERS
@@ -39,7 +47,7 @@ foldl1May, foldr1May :: Foldable t => (a -> a -> a) -> t a -> Maybe a
 foldl1May = liftMay isNull . F.foldl1
 foldr1May = liftMay isNull . F.foldr1
 
-foldl1Note, foldr1Note :: Foldable t => String -> (a -> a -> a) -> t a -> a
+foldl1Note, foldr1Note :: (Partial, Foldable t) => String -> (a -> a -> a) -> t a -> a
 foldl1Note note = fromNote note "foldl1Note on empty" .^ foldl1May
 foldr1Note note = fromNote note "foldr1Note on empty" .^ foldr1May
 
@@ -55,31 +63,31 @@ minimumDef, maximumDef :: (Foldable t, Ord a) => a -> t a -> a
 minimumDef def = fromMaybe def . minimumMay
 maximumDef def = fromMaybe def . maximumMay
 
-minimumNote, maximumNote :: (Foldable t, Ord a) => String -> t a -> a
+minimumNote, maximumNote :: (Partial, Foldable t, Ord a) => String -> t a -> a
 minimumNote note = fromNote note "minimumNote on empty" . minimumMay
 maximumNote note = fromNote note "maximumNote on empty" . maximumMay
 
-minimumByMay, maximumByMay :: (Foldable t) => (a -> a -> Ordering) -> t a -> Maybe a
+minimumByMay, maximumByMay :: Foldable t => (a -> a -> Ordering) -> t a -> Maybe a
 minimumByMay = liftMay isNull . F.minimumBy
 maximumByMay = liftMay isNull . F.maximumBy
 
-minimumByDef, maximumByDef :: (Foldable t) => a -> (a -> a -> Ordering) -> t a -> a
+minimumByDef, maximumByDef :: Foldable t => a -> (a -> a -> Ordering) -> t a -> a
 minimumByDef def = fromMaybe def .^ minimumByMay
 maximumByDef def = fromMaybe def .^ maximumByMay
 
-minimumByNote, maximumByNote :: (Foldable t) => String -> (a -> a -> Ordering) -> t a -> a
+minimumByNote, maximumByNote :: (Partial, Foldable t) => String -> (a -> a -> Ordering) -> t a -> a
 minimumByNote note = fromNote note "minimumByNote on empty" .^ minimumByMay
 maximumByNote note = fromNote note "maximumByNote on empty" .^ maximumByMay
 
 -- |
 -- > findJust op = fromJust . find op
-findJust :: Foldable t => (a -> Bool) -> t a -> a
+findJust :: (Partial, Foldable t) => (a -> Bool) -> t a -> a
 findJust = fromNote "" "findJust, no matching value" .^ F.find
 
 findJustDef :: Foldable t => a -> (a -> Bool) -> t a -> a
 findJustDef def = fromMaybe def .^ F.find
 
-findJustNote :: Foldable t => String -> (a -> Bool) -> t a -> a
+findJustNote :: (Partial, Foldable t) => String -> (a -> Bool) -> t a -> a
 findJustNote note = fromNote note "findJustNote, no matching value" .^ F.find
 
 
